/********************************************************************\
 * util.h -- utility functions that are used everywhere else for    *
 *           xacc (X-Accountant)                                    *
 * Copyright (C) 1997 Robin D. Clark                                *
 *                                                                  *
 * This program is free software; you can redistribute it and/or    *
 * modify it under the terms of the GNU General Public License as   *
 * published by the Free Software Foundation; either version 2 of   *
 * the License, or (at your option) any later version.              *
 *                                                                  *
 * This program is distributed in the hope that it will be useful,  *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of   *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the    *
 * GNU General Public License for more details.                     *
 *                                                                  *
 * You should have received a copy of the GNU General Public License*
 * along with this program; if not, write to the Free Software      *
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
 *                                                                  *
 *   Author: Rob Clark                                              *
 * Internet: rclark@cs.hmc.edu                                      *
 *  Address: 609 8th Street                                         *
 *           Huntington Beach, CA 92648-4632                        *
\********************************************************************/

#ifndef __UTIL_H__
#define __UTIL_H__

#include <Xm/Xm.h>

#define charset XmSTRING_DEFAULT_CHARSET


/** DEBUGGING MACROS ************************************************/
#include <stdio.h>

#ifdef USEDEBUG
#  define DEBUG(x)    fprintf(stderr,"debug: %s\n", x);
#  define DEBUGCMD(x) x
#  define ENTER(x)    fprintf(stderr,"Entering: %s()\n", x);
#  define LEAVE(x)    fprintf(stderr,"Leaving:  %s()\n", x);
#else
#  define DEBUG(x)    
#  define DEBUGCMD(x) 
#  define ENTER(x)
#  define LEAVE(x)
#endif

#include <errno.h>
#define ERROR()     fprintf(stderr,"%s: Line %d, error = %s\n", \
			    __FILE__, __LINE__, strerror(errno));

#ifdef DEBUGMEMORY
void   *dmalloc( size_t size );
void   dfree( void *ptr );
size_t dcoresize();
#  define _malloc(x)   dmalloc(x)
#  define _free(x)     dfree(x)
#  define _coresize()  dcoresize()
#else
#  define _malloc(x)    malloc(x)
#  define _free(x)      free(x)
#  define _coresize()   0
#endif

/** COOL MACROS *****************************************************/
#define ABS(x)   ((x)>=0) ? (x) : (-1*(x))
#define DABS(x)  ((x)>=0.0) ? (x) : (-1.0*(x))
#define isNum(x) (((x)-0x30) < 0) ? 0 : (((x)-0x30) > 9) ? 0 : 1

/** PROTOTYPES ******************************************************/
void    dateCB( Widget mw, XtPointer cd, XtPointer cb );
void    amountCB( Widget mw, XtPointer cd, XtPointer cb );
void    noeditCB( Widget mw, XtPointer cd, XtPointer cb );
void    destroyShellCB( Widget w, XtPointer cd, XtPointer cb );
void    setBusyCursor( Widget w );
void    unsetBusyCursor( Widget w );
char   *textBox( Widget parent, char *title, char *text, Boolean editable );
Boolean verifyBox( Widget parent, char *text );
void    errorBox( Widget parent, char *message );

#endif
